<?php
namespace WapplerSystems\Meilisearch\ViewHelpers;


/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use WapplerSystems\Meilisearch\System\Configuration\TypoScriptConfiguration;
use WapplerSystems\Meilisearch\Domain\Search\ResultSet\SearchResultSet;
use WapplerSystems\Meilisearch\Mvc\Controller\MeilisearchControllerContext;
use WapplerSystems\Meilisearch\ViewHelpers\AbstractMeilisearchViewHelper;
use TYPO3Fluid\Fluid\Core\Rendering\RenderingContextInterface;

/**
 * Class AbstractMeilisearchFrontendViewHelper
 *
 * @author Frans Saris <frans@beech.it>
 * @author Timo Hund <timo.hund@dkd.de>
 */
abstract class AbstractMeilisearchFrontendViewHelper extends AbstractMeilisearchViewHelper
{
    /**
     * @var MeilisearchControllerContext
     */
    protected $controllerContext;

    /**
     * @return TypoScriptConfiguration
     */
    protected function getTypoScriptConfiguration()
    {
        return $this->getControllerContext()->getTypoScriptConfiguration();
    }

    /**
     * @return SearchResultSet
     */
    protected function getSearchResultSet()
    {
        return $this->getControllerContext()->getSearchResultSet();
    }

    /**
     * @return MeilisearchControllerContext
     * @throws \InvalidArgumentException
     */
    protected function getControllerContext()
    {
        $controllerContext = null;
        if (method_exists($this->renderingContext, 'getControllerContext')) {
            $controllerContext = $this->renderingContext->getControllerContext();
        }

        if (!$controllerContext instanceof MeilisearchControllerContext) {
            throw new \InvalidArgumentException('No valid MeilisearchControllerContext found', 1512998673);
        }

        return $controllerContext;
    }

    /**
     * @param RenderingContextInterface $renderingContext
     * @return SearchResultSet
     */
    protected static function getUsedSearchResultSetFromRenderingContext(RenderingContextInterface $renderingContext)
    {
        $resultSet = $renderingContext->getVariableProvider()->get('resultSet');
        return $resultSet;
    }
}
